@@ -68,17 +68,18 @@ session_base::set_compat (bool state)
 void
 session_base::run_impl ()
 {
-  if (get_ruid() != get_uid())
-    throw error(get_ruser(), get_user(), USER_SWITCH,
+  if (get_auth()->get_ruid() != get_auth()->get_uid())
+    throw error(get_auth()->get_ruser(), get_auth()->get_user(), USER_SWITCH,
 		_("dchroot session restriction"));
 
   sbuild::session::run_impl();
 }
 
 sbuild::string_list
-session_base::get_command_directories () const
+session_base::get_command_directories (sbuild::chroot::ptr&       session_chroot,
+				       sbuild::environment const& env) const
 {
   // dchroot does not treat logins differently from commands with
   // respect to the cwd inside the chroot.
-  return get_login_directories();
+  return get_login_directories(session_chroot, env);
 }
