@@ -20,8 +20,12 @@
 
 #include "schroot-main-base.h"
 
-#include <sbuild/sbuild-auth-conv.h>
-#include <sbuild/sbuild-auth-conv-tty.h>
+#include <sbuild/sbuild-config.h>
+#ifdef SBUILD_FEATURE_PAM
+#include <sbuild/sbuild-auth-pam.h>
+#include <sbuild/sbuild-auth-pam-conv.h>
+#include <sbuild/sbuild-auth-pam-conv-tty.h>
+#endif // SBUILD_FEATURE_PAM
 
 #include <cstdlib>
 #include <ctime>
@@ -35,6 +39,7 @@
 
 using std::endl;
 using boost::format;
+using sbuild::_;
 using sbuild::N_;
 using namespace schroot;
 
@@ -59,7 +64,8 @@ namespace
       // TRANSLATORS: %1% = file
       emap(main_base::CHROOT_NOTDEFINED, N_("The specified chroots are not defined in '%1%'")),
       // TRANSLATORS: %1% = chroot name
-      emap(main_base::CHROOT_NOTFOUND,   N_("%1%: Chroot not found"))
+      emap(main_base::CHROOT_NOTFOUND,   N_("%1%: Chroot not found")),
+      emap(main_base::SESSION_INVALID,   N_("%1%: Invalid session name"))
     };
 
 }
@@ -86,6 +92,33 @@ main_base::~main_base ()
 {
 }
 
+void
+main_base::action_version (std::ostream& stream)
+{
+  schroot_base::main::action_version(stream);
+
+  format feature("  %1$-12s %2%\n");
+
+  stream << '\n'
+	 << _("Available chroot types:") << '\n';
+#ifdef SBUILD_FEATURE_BLOCKDEV
+  stream << feature % "BLOCKDEV" % _("Support for 'block-device' chroots");
+#endif
+#ifdef SBUILD_FEATURE_BTRFSSNAP
+  stream << feature % "BTRFSSNAP" % _("Support for 'btrfs-snapshot' chroots");
+#endif
+  stream << feature % "DIRECTORY" % _("Support for 'directory' chroots");
+  stream << feature % "FILE" % _("Support for 'file' chroots");
+#ifdef SBUILD_FEATURE_LOOPBACK
+  stream << feature % "LOOPBACK" % _("Support for 'loopback' chroots");
+#endif
+#ifdef SBUILD_FEATURE_LVMSNAP
+  stream << feature % "LVMSNAP" % _("Support for 'lvm-snapshot' chroots");
+#endif
+  stream << feature % "PLAIN" % _("Support for 'plain' chroots");
+  stream << std::flush;
+}
+
 void
 main_base::action_info ()
 {
@@ -232,6 +265,9 @@ main_base::run_impl ()
 	}
     }
 
+  if (this->config->find_alias(this->options->session_name))
+    throw error(this->options->session_name, SESSION_INVALID);
+
   /* Print chroot information for specified chroots. */
   if (this->options->action == options_base::ACTION_INFO)
     {
@@ -263,30 +299,19 @@ main_base::run_impl ()
   try
     {
       create_session(sess_op);
+      add_session_auth();
 
       if (!this->options->command.empty())
-	this->session->set_command(this->options->command);
+	this->session->get_auth()->set_command(this->options->command);
       if (!this->options->directory.empty())
-	this->session->set_wd(this->options->directory);
-      if (this->options->preserve)
-	this->session->set_environment(environ);
+	this->session->get_auth()->set_wd(this->options->directory);
+      this->session->set_preserve_environment(this->options->preserve);
       this->session->set_session_id(this->options->session_name);
       this->session->set_force(this->options->session_force);
-      sbuild::auth::verbosity verbosity = sbuild::auth::VERBOSITY_NORMAL;
       if (this->options->quiet)
-	verbosity = sbuild::auth::VERBOSITY_QUIET;
+	this->session->set_verbosity("quiet");
       else if (this->options->verbose)
-	verbosity = sbuild::auth::VERBOSITY_VERBOSE;
-      this->session->set_verbosity(verbosity);
-
-      /* Set up authentication timeouts. */
-      std::tr1::shared_ptr<sbuild::auth_conv>
-	conv(new sbuild::auth_conv_tty);
-      time_t curtime = 0;
-      time(&curtime);
-      conv->set_warning_timeout(curtime + 15);
-      conv->set_fatal_timeout(curtime + 20);
-      this->session->set_conv(conv);
+	this->session->set_verbosity("verbose");
 
       /* Run session. */
       this->session->run();
@@ -302,3 +327,29 @@ main_base::run_impl ()
   else
     return EXIT_FAILURE;
 }
+
+void
+main_base::add_session_auth ()
+{
+  // Add PAM authentication handler.  If PAM isn't available, just
+  // continue to use the default handler
+
+#ifdef SBUILD_FEATURE_PAM
+  sbuild::auth::ptr auth = sbuild::auth_pam::create("schroot");
+
+  sbuild::auth_pam_conv::auth_ptr pam_auth =
+    std::tr1::dynamic_pointer_cast<sbuild::auth_pam>(auth);
+
+  sbuild::auth_pam_conv::ptr conv = sbuild::auth_pam_conv_tty::create(pam_auth);
+
+
+  /* Set up authentication timeouts. */
+  time_t curtime = 0;
+  time(&curtime);
+  conv->set_warning_timeout(curtime + 15);
+  conv->set_fatal_timeout(curtime + 20);
+  pam_auth->set_conv(conv);
+
+  this->session->set_auth(auth);
+#endif // SBUILD_FEATURE_PAM
+}
