@@ -42,7 +42,8 @@ namespace schroot
 	CHROOT_FILE,       ///< No chroots are defined in ....
 	CHROOT_FILE2,      ///< No chroots are defined in ... or ....
 	CHROOT_NOTDEFINED, ///< The specified chroots are not defined.
-	CHROOT_NOTFOUND    ///< Chroot not found.
+	CHROOT_NOTFOUND,   ///< Chroot not found.
+	SESSION_INVALID    ///< Invalid session name.
       };
 
     /// Exception type.
@@ -65,6 +66,9 @@ namespace schroot
     /// The destructor.
     virtual ~main_base ();
 
+    virtual void
+    action_version (std::ostream& stream);
+
     /**
      * List chroots.
      */
@@ -129,6 +133,12 @@ namespace schroot
     virtual void
     create_session (sbuild::session::operation sess_op) = 0;
 
+    /**
+     * Add PAM authentication handler to the session.
+     */
+    virtual void
+    add_session_auth ();
+
   protected:
     /// The program options.
     options_base::ptr          options;
