@@ -20,6 +20,7 @@
 
 #include "sbuild-chroot.h"
 #include "sbuild-chroot-facet-source-clonable.h"
+#include "sbuild-chroot-facet-session-clonable.h"
 #include "sbuild-chroot-config.h"
 #include "sbuild-lock.h"
 
@@ -80,13 +81,15 @@ error<chroot_config::error_code>::error_strings
  init_errors + (sizeof(init_errors) / sizeof(init_errors[0])));
 
 chroot_config::chroot_config ():
-  chroots()
+  chroots(),
+  aliases()
 {
 }
 
 chroot_config::chroot_config (std::string const& file,
 			      bool               active):
-  chroots()
+  chroots(),
+  aliases()
 {
   add(file, active);
 }
@@ -515,9 +518,19 @@ chroot_config::load_keyfile (keyfile& kconfig,
       // get the correct facets in place.  In the future, it would be
       // great if sessions could serialise their facet usage to allow
       // automatic reconstruction.
-      if (active)
+      chroot_facet_session_clonable::const_ptr clonable
+	(chroot->get_facet<chroot_facet_session_clonable>());
+
+      log_debug(DEBUG_INFO) << "Created template chroot (type=" << type
+			    << "  name/session-id=" << *group
+			    << "  active=" << active
+			    << "  source-clonable=" << static_cast<bool>(clonable)
+			    << ")" << endl;
+
+      if (active && clonable)
 	{
 	  chroot = chroot->clone_session("dummy-session-name", "", false);
+	  assert(chroot);
 	  assert(chroot->get_active());
 	}
       else
@@ -536,7 +549,7 @@ chroot_config::load_keyfile (keyfile& kconfig,
 	chroot_facet_source_clonable::const_ptr psrc
 	  (chroot->get_facet<sbuild::chroot_facet_source_clonable>());
 
-	if (psrc && !chroot->get_active())
+	if (psrc && psrc->get_source_clone() && !chroot->get_active())
 	  {
 	    chroot::ptr source_chroot = chroot->clone_source();
 	    if (source_chroot)
