@@ -62,28 +62,37 @@ namespace sbuild
 	SESSION_PURGE   = 1 << 2  ///< The chroot should be purged.
       };
 
+    /// Message verbosity
+    enum verbosity
+      {
+	VERBOSITY_QUIET,  ///< Only print essential messages.
+	VERBOSITY_NORMAL, ///< Print messages (the default).
+	VERBOSITY_VERBOSE ///< Print all messages.
+      };
+
     /// Error codes.
     enum error_code
       {
-	CHROOT_CREATE,   ///< Chroot creation failed.
-	CHROOT_DEVICE,   ///< Chroot device name not set.
-	CHROOT_TYPE,     ///< Unknown chroot type.
-	DEVICE_ABS,      ///< Device must have an absolute path.
-	DEVICE_LOCK,     ///< Failed to lock device.
-	DEVICE_NOTBLOCK, ///< File is not a block device.
-	DEVICE_UNLOCK,   ///< Failed to unlock device.
-	DIRECTORY_ABS,   ///< Directory must have an absolute path.
-	FACET_INVALID,   ///< Attempt to add object which is not a facet.
-	FACET_PRESENT,   ///< Attempt to add facet which is already in use.
-	FILE_ABS,        ///< File must have an absolute path.
-	FILE_LOCK,       ///< Failed to acquire lock.
-	FILE_NOTREG,     ///< File is not a regular file.
-	FILE_OWNER,      ///< File is not owned by user root.
-	FILE_PERMS,      ///< File has write permissions for others.
-	FILE_UNLOCK,     ///< Failed to discard lock.
-	LOCATION_ABS,    ///< Location must have an absolute path.
-	SESSION_UNLINK,  ///< Failed to unlink session file.
-	SESSION_WRITE    ///< Failed to write session file.
+	CHROOT_CREATE,    ///< Chroot creation failed.
+	CHROOT_DEVICE,    ///< Chroot device name not set.
+	CHROOT_TYPE,      ///< Unknown chroot type.
+	DEVICE_ABS,       ///< Device must have an absolute path.
+	DEVICE_LOCK,      ///< Failed to lock device.
+	DEVICE_NOTBLOCK,  ///< File is not a block device.
+	DEVICE_UNLOCK,    ///< Failed to unlock device.
+	DIRECTORY_ABS,    ///< Directory must have an absolute path.
+	FACET_INVALID,    ///< Attempt to add object which is not a facet.
+	FACET_PRESENT,    ///< Attempt to add facet which is already in use.
+	FILE_ABS,         ///< File must have an absolute path.
+	FILE_LOCK,        ///< Failed to acquire lock.
+	FILE_NOTREG,      ///< File is not a regular file.
+	FILE_OWNER,       ///< File is not owned by user root.
+	FILE_PERMS,       ///< File has write permissions for others.
+	FILE_UNLOCK,      ///< Failed to discard lock.
+	LOCATION_ABS,     ///< Location must have an absolute path.
+	SESSION_UNLINK,   ///< Failed to unlink session file.
+	SESSION_WRITE,    ///< Failed to write session file.
+	VERBOSITY_INVALID ///< Message verbosity is invalid.
       };
 
     /// Exception type.
@@ -342,6 +351,22 @@ namespace sbuild
     void
     set_aliases (string_list const& aliases);
 
+    /**
+     * Check if the environment should be preserved in the chroot.
+     *
+     * @returns true to preserve or false to clean.
+     */
+    bool
+    get_preserve_environment () const;
+
+    /**
+     * Set if the environment should be preserved in the chroot.
+     *
+     * @param preserve_environment true to preserve or false to clean.
+     */
+    void
+    set_preserve_environment (bool preserve_environment);
+
     /**
      * Get the environment filter of the chroot.  This is a POSIX
      * extended regular expression used to remove insecure environment
@@ -353,7 +378,7 @@ namespace sbuild
     get_environment_filter () const;
 
     /**
-     * Get the environment filter of the chroot.  This is a POSIX
+     * Set the environment filter of the chroot.  This is a POSIX
      * extended regular expression used to remove insecure environment
      * variables from the chroot environment.
      *
@@ -444,6 +469,38 @@ namespace sbuild
     void
     set_command_prefix (string_list const& command_prefix);
 
+    /**
+     * Get the message verbosity.
+     *
+     * Returns the verbosity level.
+     */
+    verbosity
+    get_verbosity () const;
+
+    /**
+     * Get the message verbosity.
+     *
+     * Returns the verbosity level.
+     */
+    const char *
+    get_verbosity_string () const;
+
+    /**
+     * Set the message verbosity.
+     *
+     * @param verbosity the verbosity level.
+     */
+    void
+    set_verbosity (verbosity verbosity);
+
+    /**
+     * Set the message verbosity.
+     *
+     * @param verbosity the verbosity level.
+     */
+    void
+    set_verbosity (std::string const& verbosity);
+
     /**
      * Get the type of the chroot.
      *
@@ -719,6 +776,8 @@ namespace sbuild
     string_list   root_groups;
     /// Alternative names for the chroot.
     string_list   aliases;
+    /// Preserve environment?
+    bool          preserve_environment;
     /// Environment filter regex.
     regex         environment_filter;
     /// Location to mount chroot in the filesystem (if any).
@@ -731,9 +790,12 @@ namespace sbuild
     std::string   script_config;
     /// Command prefix.
     string_list   command_prefix;
+    /// The message verbosity.
+    verbosity     message_verbosity;
 
     typedef std::tr1::shared_ptr<chroot_facet> facet_ptr;
     typedef std::list<facet_ptr> facet_list;
+    /// Contained chroot facets
     facet_list facets;
   };
 
