@@ -1,4 +1,4 @@
-/* Copyright © 2005-2009  Roger Leigh <rleigh@debian.org>
+/* Copyright © 2005-2010  Roger Leigh <rleigh@debian.org>
  *
  * schroot is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,7 +22,13 @@
 #include "sbuild-chroot-facet-personality.h"
 #include "sbuild-chroot-facet-session.h"
 #include "sbuild-chroot-facet-session-clonable.h"
+#ifdef SBUILD_FEATURE_PAM
+#include "sbuild-auth-pam.h"
+#include "sbuild-auth-pam-conv.h"
+#include "sbuild-auth-pam-conv-tty.h"
+#else
 #include "sbuild-auth-null.h"
+#endif // SBUILD_FEATURE_PAM
 #include "sbuild-ctty.h"
 #include "sbuild-run-parts.h"
 #include "sbuild-session.h"
@@ -76,9 +82,9 @@ namespace
       // TRANSLATORS: %1% = chroot name
       emap(session::CHROOT_ALIAS,   N_("No chroot found matching name or alias '%1%'")),
       emap(session::CHROOT_LOCK,    N_("Failed to lock chroot")),
+      emap(session::CHROOT_NOTFOUND,N_("%1%: Chroot not found")),
       emap(session::CHROOT_SETUP,   N_("Chroot setup failed")),
       // TRANSLATORS: %1% = chroot name
-      emap(session::CHROOT_UNKNOWN, N_("Failed to find chroot '%1%'")),
       emap(session::CHROOT_UNLOCK,  N_("Failed to unlock chroot")),
       // TRANSLATORS: %1% = command
       emap(session::COMMAND_ABS,    N_("Command \"%1%\" must have an absolute path")),
@@ -178,6 +184,7 @@ namespace
   }
 
   volatile bool sighup_called = false;
+  volatile bool sigint_called = false;
   volatile bool sigterm_called = false;
 
   /**
@@ -192,6 +199,30 @@ namespace
     sighup_called = true;
   }
 
+  /**
+   * Handle the SIGINT signal.
+   *
+   * We explicitly do nothing with SIGINT, and rely on the exit status
+   * of child processes to determine if we should care.  We want to
+   * make sure any child process (which will also have received
+   * SIGINT) has exited before we do anything, and some child
+   * processes (for example, emacs) may expect SIGINT during normal
+   * operation.  See http://www.cons.org/cracauer/sigint.html for a
+   * good discussion of SIGINT handling.
+
+   * @param ignore the signal number.
+   */
+  void
+  sigint_handler (int ignore)
+  {
+    /*
+     * Allows us to detect if an interrupted waitpid() was interrupted
+     * due to SIGINT or something else.  We may also want to use this
+     * at exit time to see if we should re-kill ourselves with SIGINT.
+     */
+    sigint_called = true;
+  }
+
   /**
    * Handle the SIGTERM signal.
    *
@@ -220,7 +251,13 @@ session::session (std::string const&         service,
 		  config_ptr&                config,
 		  operation                  operation,
 		  sbuild::string_list const& chroots):
-  authstat(auth_null::create(service)),
+  authstat(
+#ifdef SBUILD_FEATURE_PAM
+	   auth_pam::create(service)
+#else
+	   auth_null::create(service)
+#endif // SBUILD_FEATURE_PAM
+	   ),
   config(config),
   chroots(chroots),
   chroot_status(true),
@@ -230,9 +267,12 @@ session::session (std::string const&         service,
   session_id(),
   force(false),
   saved_sighup_signal(),
+  saved_sigint_signal(),
   saved_sigterm_signal(),
   saved_termios(),
   termios_ok(false),
+  verbosity(),
+  preserve_environment(false),
   cwd(sbuild::getcwd())
 {
 }
@@ -301,6 +341,30 @@ session::set_session_id (std::string const& session_id)
   this->session_id = session_id;
 }
 
+std::string const&
+session::get_verbosity () const
+{
+  return this->verbosity;
+}
+
+void
+session::set_verbosity (std::string const& verbosity)
+{
+  this->verbosity = verbosity;
+}
+
+bool
+session::get_preserve_environment () const
+{
+  return this->preserve_environment;
+}
+
+void
+session::set_preserve_environment (bool preserve_environment)
+{
+  this->preserve_environment = preserve_environment;
+}
+
 bool
 session::get_force () const
 {
@@ -559,6 +623,8 @@ session::run_impl ()
     {
       sighup_called = false;
       set_sighup_handler();
+      sigint_called = false;
+      set_sigint_handler();
       sigterm_called = false;
       set_sigterm_handler();
 
@@ -572,7 +638,14 @@ session::run_impl ()
 
 	  const chroot::ptr ch = this->config->find_alias(*cur);
 	  if (!ch) // Should never happen, but cater for it anyway.
-	    throw error(*cur, CHROOT_UNKNOWN);
+	    throw error(*cur, CHROOT_NOTFOUND);
+
+	  // TODO: Make chroot/session selection automatically fail
+	  // if no session exists earlier on when selecting chroots.
+	  if (ch->get_session_flags() & chroot::SESSION_CREATE &&
+	      (this->session_operation != OPERATION_AUTOMATIC &&
+	       this->session_operation != OPERATION_BEGIN))
+	      throw error(*cur, CHROOT_NOTFOUND);
 
 	  // For now, use a copy of the chroot; if we create a session
 	  // later, we will replace it.
@@ -619,6 +692,10 @@ session::run_impl ()
 	    }
 	  assert(chroot);
 
+	  // Override chroot verbosity if needed.
+	  if (!this->verbosity.empty())
+	    chroot->set_verbosity(this->verbosity);
+
 	  // Following authentication success, default child status to
 	  // success so that operations such as beginning, ending and
 	  // recovering sessions will return success unless an
@@ -683,6 +760,7 @@ session::run_impl ()
   catch (error const& e)
     {
       clear_sigterm_handler();
+      clear_sigint_handler();
       clear_sighup_handler();
 
       /* If a command was not run, but something failed, the exit
@@ -693,11 +771,13 @@ session::run_impl ()
     }
 
   clear_sigterm_handler();
+  clear_sigint_handler();
   clear_sighup_handler();
 }
 
 string_list
-session::get_login_directories () const
+session::get_login_directories (sbuild::chroot::ptr& session_chroot,
+				environment const&   env) const
 {
   string_list ret;
 
@@ -713,7 +793,6 @@ session::get_login_directories () const
       ret.push_back(this->cwd);
 
       // Set $HOME.
-      environment env = this->authstat->get_auth_environment();
       std::string home;
       if (env.get("HOME", home) &&
 	  std::find(ret.begin(), ret.end(), home) == ret.end())
@@ -732,7 +811,8 @@ session::get_login_directories () const
 }
 
 string_list
-session::get_command_directories () const
+session::get_command_directories (sbuild::chroot::ptr& session_chroot,
+				  environment const&   env) const
 {
   string_list ret;
 
@@ -776,14 +856,15 @@ session::get_shell () const
 void
 session::get_command (sbuild::chroot::ptr& session_chroot,
 		      std::string&         file,
-		      string_list&         command) const
+		      string_list&         command,
+		      environment const&   env) const
 {
   /* Run login shell */
   if (command.empty() ||
       command[0].empty()) // No command
     get_login_command(session_chroot, file, command);
   else
-    get_user_command(session_chroot, file, command);
+    get_user_command(session_chroot, file, command, env);
 }
 
 void
@@ -796,9 +877,13 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
   std::string shell = get_shell();
   file = shell;
 
-  if (this->authstat->get_environment().empty() &&
-      session_chroot->get_command_prefix().empty())
+  bool login_shell =
+    !(get_preserve_environment() ||
+      session_chroot->get_preserve_environment()) &&
+    session_chroot->get_command_prefix().empty();
+
     // Not keeping environment and can setup argv correctly; login shell
+    if (login_shell)
     {
       std::string shellbase = basename(shell);
       std::string loginshell = "-" + shellbase;
@@ -826,13 +911,12 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
 	       shell.c_str());
     }
 
-  if (this->authstat->get_verbosity() == auth::VERBOSITY_VERBOSE)
+  if (session_chroot->get_verbosity() == chroot::VERBOSITY_VERBOSE)
     {
       std::string format_string;
       if (this->authstat->get_ruid() == this->authstat->get_uid())
 	{
-	  if (this->authstat->get_environment().empty() &&
-	      session_chroot->get_command_prefix().empty())
+	  if (login_shell)
 	    // TRANSLATORS: %1% = chroot name
 	    // TRANSLATORS: %4% = command
 	    format_string = _("[%1% chroot] Running login shell: '%4%'");
@@ -843,8 +927,7 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
 	}
       else
 	{
-	  if (this->authstat->get_environment().empty() &&
-	      session_chroot->get_command_prefix().empty())
+	  if (login_shell)
 	    // TRANSLATORS: %1% = chroot name
 	    // TRANSLATORS: %2% = user name
 	    // TRANSLATORS: %3% = user name
@@ -871,10 +954,10 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
 void
 session::get_user_command (sbuild::chroot::ptr& session_chroot,
 			   std::string&         file,
-			   string_list&         command) const
+			   string_list&         command,
+			   environment const&   env) const
 {
   /* Search for program in path. */
-  environment env = this->authstat->get_auth_environment();
   std::string path;
   if (!env.get("PATH", path))
     path.clear();
@@ -889,7 +972,7 @@ session::get_user_command (sbuild::chroot::ptr& session_chroot,
     syslog(LOG_USER|LOG_NOTICE, "[%s chroot] (%s->%s) Running command: \"%s\"",
 	   session_chroot->get_name().c_str(), this->authstat->get_ruser().c_str(), this->authstat->get_user().c_str(), commandstring.c_str());
 
-  if (this->authstat->get_verbosity() == auth::VERBOSITY_VERBOSE)
+  if (session_chroot->get_verbosity() == chroot::VERBOSITY_VERBOSE)
     {
       std::string format_string;
       if (this->authstat->get_ruid() == this->authstat->get_uid())
@@ -997,38 +1080,26 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
   env.add("AUTH_RGID", this->authstat->get_rgid());
   env.add("AUTH_HOME", this->authstat->get_home());
   env.add("AUTH_SHELL", this->authstat->get_shell());
-  {
-    const char *verbosity = 0;
-    switch (this->authstat->get_verbosity())
-      {
-      case auth::VERBOSITY_QUIET:
-	verbosity = "quiet";
-	break;
-      case auth::VERBOSITY_NORMAL:
-	verbosity = "normal";
-	break;
-      case auth::VERBOSITY_VERBOSE:
-	verbosity = "verbose";
-	break;
-      default:
-	log_debug(DEBUG_CRITICAL) << format("Invalid verbosity level: %1%, falling back to 'normal'")
-	  % static_cast<int>(this->authstat->get_verbosity())
-		     << endl;
-	verbosity = "normal";
-	break;
-      }
-    env.add("AUTH_VERBOSITY", verbosity);
-  }
 
+  env.add("AUTH_VERBOSITY", session_chroot->get_verbosity_string());
+  env.add("VERBOSE", session_chroot->get_verbosity_string());
   env.add("MOUNT_DIR", SCHROOT_MOUNT_DIR);
   env.add("LIBEXEC_DIR", SCHROOT_LIBEXEC_DIR);
+  env.add("SYSCONF_DIR", SCHROOT_SYSCONF_DIR);
+  env.add("DATA_DIR", SCHROOT_DATA_DIR);
+  env.add("SETUP_DATA_DIR", SCHROOT_SETUP_DATA_DIR);
   env.add("PID", getpid());
   env.add("SESSION_ID", session_chroot->get_session_id());
+  env.add("HOST", SBUILD_HOST);
+  env.add("HOST_OS", SBUILD_HOST_OS);
+  env.add("HOST_VENDOR", SBUILD_HOST_VENDOR);
+  env.add("HOST_CPU", SBUILD_HOST_CPU);
+  env.add("PLATFORM", SBUILD_PLATFORM);
 
   run_parts rp(SCHROOT_CONF_SETUP_D,
 	       true, true, 022);
   rp.set_reverse(setup_type == chroot::SETUP_STOP);
-  rp.set_verbose(this->authstat->get_verbosity() == auth::VERBOSITY_VERBOSE);
+  rp.set_verbose(session_chroot->get_verbosity() == chroot::VERBOSITY_VERBOSE);
 
   log_debug(DEBUG_INFO) << rp << std::endl;
 
@@ -1105,6 +1176,15 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
   assert(!get_shell().empty());
   assert(this->authstat->is_initialised()); // PAM must be initialised
 
+  /* Set up environment */
+  environment env;
+  env.set_filter(session_chroot->get_environment_filter());
+
+  if (get_preserve_environment() || session_chroot->get_preserve_environment())
+    env += this->authstat->get_complete_environment();
+  else
+    env += this->authstat->get_auth_environment();
+
   // Store before chroot call.
   this->cwd = sbuild::getcwd();
   log_debug(DEBUG_INFO) << "CWD=" << this->cwd << std::endl;
@@ -1158,9 +1238,9 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
   string_list dlist;
   if (command.empty() ||
       command[0].empty()) // No command
-    dlist = get_login_directories();
+    dlist = get_login_directories(session_chroot, env);
   else
-    dlist = get_command_directories();
+    dlist = get_command_directories(session_chroot, env);
   log_debug(DEBUG_INFO)
     << format("Directory fallbacks: %1%") % string_list_to_string(dlist, ", ") << endl;
 
@@ -1192,16 +1272,11 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
     }
 
   /* Fix up the command for exec. */
-  get_command(session_chroot, file, command);
+  get_command(session_chroot, file, command, env);
   log_debug(DEBUG_NOTICE) << "command="
 			  << string_list_to_string(command, ", ")
 			  << std::endl;
 
-  /* Set up environment */
-  environment env;
-  env.set_filter(session_chroot->get_environment_filter());
-  env += this->authstat->get_auth_environment();
-
   // Add equivalents to sudo's SUDO_USER, SUDO_UID, SUDO_GID, and
   // SUDO_COMMAND.
   env.add(std::make_pair("SCHROOT_COMMAND",
@@ -1251,6 +1326,15 @@ session::wait_for_child (pid_t pid,
 
   while (1)
     {
+      /*
+       * If we (the parent process) gets SIGHUP or SIGTERM, pass this
+       * signal on to the child (once).  Note that we do not handle
+       * SIGINT this way, because when a user presses Ctrl-C, the
+       * SIGINT is sent to all processes attached to that TTY (so the
+       * child will already have gotten it).  In any case, once the
+       * child gets the signal, we just have to continue waiting for
+       * it to exit.
+       */
       if ((sighup_called || sigterm_called) && !child_killed)
 	{
 	  if (sighup_called)
@@ -1273,7 +1357,7 @@ session::wait_for_child (pid_t pid,
 
       if (waitpid(pid, &status, 0) == -1)
 	{
-	  if (errno == EINTR && (sighup_called || sigterm_called))
+	  if (errno == EINTR && (sighup_called || sigterm_called || sigint_called))
 	    continue; // Kill child and wait again.
 	  else
 	    throw error(CHILD_WAIT, strerror(errno));
@@ -1288,6 +1372,8 @@ session::wait_for_child (pid_t pid,
 	  sigterm_called = false;
 	  throw error(SIGNAL_CATCH, strsignal(SIGTERM));
 	}
+      // No need to handle the SIGINT case here; it is handled
+      // correctly below
       else
 	break;
     }
@@ -1354,6 +1440,18 @@ session::clear_sighup_handler ()
   clear_signal_handler(SIGHUP, &this->saved_sighup_signal);
 }
 
+void
+session::set_sigint_handler ()
+{
+  set_signal_handler(SIGINT, &this->saved_sigint_signal, sigint_handler);
+}
+
+void
+session::clear_sigint_handler ()
+{
+  clear_signal_handler(SIGINT, &this->saved_sigint_signal);
+}
+
 void
 session::set_sigterm_handler ()
 {
