@@ -44,13 +44,40 @@ public:
   clone () const
   { return ptr(new basic_chroot(*this)); }
 
+  virtual ptr
+  clone_session (std::string const& session_id,
+		 std::string const& user,
+		 bool               root) const
+  { return ptr(); }
+
+
+  chroot::ptr
+  clone_source () const
+  { return ptr(); }
+
   virtual std::string const&
   get_chroot_type () const
   { static const std::string type("test"); return type; }
 
+  void
+  set_run_setup_scripts (bool run_setup_scripts)
+  {
+    sbuild::chroot::set_run_setup_scripts(run_setup_scripts);
+  }
+
+  virtual std::string
+  get_path () const
+  { return get_mount_location(); }
+
   virtual void
-  setup_env (sbuild::environment& env)
-  { this->sbuild::chroot::setup_env(env); }
+  setup_env (sbuild::chroot const& chroot,
+	     sbuild::environment&  env) const
+  { sbuild::chroot::setup_env(chroot, env); }
+
+  virtual void
+  get_details (sbuild::chroot const&  chroot,
+	       sbuild::format_detail& detail) const
+  { sbuild::chroot::get_details(chroot, detail); }
 
   virtual void
   setup_lock (setup_type type,
@@ -59,13 +86,19 @@ public:
   {}
 
   virtual sbuild::chroot::session_flags
-  get_session_flags () const
+  get_session_flags (sbuild::chroot const& chroot) const
   { return sbuild::chroot::SESSION_CREATE; }
 
   virtual void
-  print_details (std::ostream& stream) const
-  { sbuild::chroot::print_details(stream); }
+  get_keyfile (sbuild::chroot const& chroot,
+	       sbuild::keyfile&      keyfile) const
+  { sbuild::chroot::get_keyfile(chroot, keyfile); }
 
+  virtual void
+  set_keyfile (sbuild::chroot&        chroot,
+	       sbuild::keyfile const& keyfile,
+	       sbuild::string_list&   used_keys)
+  { sbuild::chroot::set_keyfile(chroot, keyfile, used_keys); }
 };
 
 class test_chroot : public test_chroot_base<basic_chroot>
@@ -73,7 +106,6 @@ class test_chroot : public test_chroot_base<basic_chroot>
   CPPUNIT_TEST_SUITE(test_chroot);
   CPPUNIT_TEST(test_name);
   CPPUNIT_TEST(test_description);
-  CPPUNIT_TEST(test_mount_device);
   CPPUNIT_TEST(test_mount_location);
   CPPUNIT_TEST(test_priority);
   CPPUNIT_TEST(test_groups);
@@ -82,9 +114,12 @@ class test_chroot : public test_chroot_base<basic_chroot>
   CPPUNIT_TEST(test_environment_filter);
   CPPUNIT_TEST(test_active);
   CPPUNIT_TEST(test_run_setup_scripts);
-  CPPUNIT_TEST(test_run_exec_scripts);
+  CPPUNIT_TEST(test_verbose);
+  CPPUNIT_TEST(test_preserve_environment);
+  CPPUNIT_TEST_EXCEPTION(test_verbose_error, sbuild::chroot::error);
   CPPUNIT_TEST(test_chroot_type);
   CPPUNIT_TEST(test_setup_env);
+  CPPUNIT_TEST(test_setup_keyfile);
   CPPUNIT_TEST(test_session_flags);
   CPPUNIT_TEST(test_print_details);
   CPPUNIT_TEST(test_print_config);
@@ -114,13 +149,6 @@ public:
 		   "/mnt/mount-location/example");
   }
 
-  void test_mount_device()
-  {
-    chroot->set_mount_device("/dev/device-to-mount/example");
-    CPPUNIT_ASSERT(chroot->get_mount_device() ==
-		   "/dev/device-to-mount/example");
-  }
-
   void test_priority()
   {
     chroot->set_priority(6);
@@ -178,28 +206,47 @@ public:
   void test_active()
   {
     CPPUNIT_ASSERT(chroot->get_active() == false);
-    chroot->set_active(true);
-    CPPUNIT_ASSERT(chroot->get_active() == true);
-    chroot->set_active(false);
-    CPPUNIT_ASSERT(chroot->get_active() == false);
   }
 
   void test_run_setup_scripts()
   {
-    CPPUNIT_ASSERT(chroot->get_run_setup_scripts() == false);
-    chroot->set_run_setup_scripts(true);
+    std::tr1::shared_ptr<basic_chroot> c = std::tr1::dynamic_pointer_cast<basic_chroot>(chroot);
+
     CPPUNIT_ASSERT(chroot->get_run_setup_scripts() == true);
-    chroot->set_run_setup_scripts(false);
+    c->set_run_setup_scripts(false);
     CPPUNIT_ASSERT(chroot->get_run_setup_scripts() == false);
+    c->set_run_setup_scripts(true);
+    CPPUNIT_ASSERT(chroot->get_run_setup_scripts() == true);
   }
 
-  void test_run_exec_scripts()
+  void test_verbose()
   {
-    CPPUNIT_ASSERT(chroot->get_run_exec_scripts() == false);
-    chroot->set_run_exec_scripts(true);
-    CPPUNIT_ASSERT(chroot->get_run_exec_scripts() == true);
-    chroot->set_run_exec_scripts(false);
-    CPPUNIT_ASSERT(chroot->get_run_exec_scripts() == false);
+    std::tr1::shared_ptr<basic_chroot> c = std::tr1::dynamic_pointer_cast<basic_chroot>(chroot);
+
+    CPPUNIT_ASSERT(chroot->get_verbosity() == sbuild::chroot::VERBOSITY_QUIET);
+    c->set_verbosity(sbuild::chroot::VERBOSITY_VERBOSE);
+    CPPUNIT_ASSERT(chroot->get_verbosity() == sbuild::chroot::VERBOSITY_VERBOSE);
+    CPPUNIT_ASSERT(std::string(chroot->get_verbosity_string()) == "verbose");
+    c->set_verbosity("normal");
+    CPPUNIT_ASSERT(chroot->get_verbosity() == sbuild::chroot::VERBOSITY_NORMAL);
+    CPPUNIT_ASSERT(std::string(chroot->get_verbosity_string()) == "normal");
+  }
+
+  void test_preserve_environment()
+  {
+    std::tr1::shared_ptr<basic_chroot> c = std::tr1::dynamic_pointer_cast<basic_chroot>(chroot);
+
+    CPPUNIT_ASSERT(chroot->get_preserve_environment() == false);
+    c->set_preserve_environment(true);
+    CPPUNIT_ASSERT(chroot->get_preserve_environment() == true);
+    c->set_preserve_environment(false);
+    CPPUNIT_ASSERT(chroot->get_preserve_environment() == false);
+  }
+
+  void test_verbose_error()
+  {
+    std::tr1::shared_ptr<basic_chroot> c = std::tr1::dynamic_pointer_cast<basic_chroot>(chroot);
+    c->set_verbosity("invalid");
   }
 
   void test_chroot_type()
@@ -210,18 +257,29 @@ public:
   void test_setup_env()
   {
     sbuild::environment expected;
+    setup_env_chroot(expected);
     expected.add("CHROOT_TYPE",           "test");
-    expected.add("CHROOT_NAME",           "test-name");
-    expected.add("CHROOT_DESCRIPTION",    "test-description");
     expected.add("CHROOT_MOUNT_LOCATION", "/mnt/mount-location");
     expected.add("CHROOT_PATH",           "/mnt/mount-location");
-    expected.add("CHROOT_MOUNT_DEVICE",   "/dev/device-to-mount");
-    expected.add("CHROOT_SCRIPT_CONFIG",  sbuild::normalname(std::string(PACKAGE_SYSCONF_DIR) + "/script-defaults"));
-    expected.add("CHROOT_SESSION_CLONE", "false");
+    expected.add("CHROOT_SESSION_CLONE",  "false");
     expected.add("CHROOT_SESSION_CREATE", "true");
-    expected.add("CHROOT_SESSION_PURGE", "false");
+    expected.add("CHROOT_SESSION_PURGE",  "false");
 
-    test_chroot_base<basic_chroot>::test_setup_env(expected);
+    sbuild::environment observed;
+    chroot->setup_env(observed);
+
+    test_chroot_base<basic_chroot>::test_setup_env(observed, expected);
+  }
+
+  void test_setup_keyfile()
+  {
+    sbuild::keyfile expected;
+    std::string group = chroot->get_name();
+    setup_keyfile_chroot(expected, group);
+    expected.set_value(group, "type", "test");
+
+    test_chroot_base<basic_chroot>::test_setup_keyfile
+      (chroot, expected, group);
   }
 
   void test_session_flags()
